/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package org.glassfish.scripting.grails;

import org.glassfish.api.admin.ServerEnvironment;
import org.glassfish.api.deployment.DeployCommandParameters;
import org.glassfish.api.deployment.Deployer;
import org.glassfish.api.deployment.DeploymentContext;
import org.glassfish.api.deployment.MetaData;
import org.glassfish.api.deployment.archive.ReadableArchive;
import org.jvnet.hk2.annotations.Inject;
import org.jvnet.hk2.annotations.Service;

import java.io.File;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.logging.Level;

/**
 * @author Vivek Pandey
 */
@Service
public class GrailsDeployer implements Deployer<GrailsContainer, GrailsApplicationContainer> {

    @Inject
    ServerEnvironment env;

    private final String root = System.getProperty("com.sun.aas.installRoot");

    public MetaData getMetaData() {
        return new MetaData(true,null, null);
    }

    public <V> V loadMetaData(Class<V> vClass, DeploymentContext deploymentContext) {

        /**
         * Try to locate Grails installation, that has glassfish-grails.jar
         */
        String grailsHome = System.getenv().get("GRAILS_HOME");
        if (grailsHome == null) {
            //try system property grails.home
            grailsHome = System.getProperty("grails.home");
        }

        if (grailsHome == null) {
            //now try to look for grails IPS package
            if (root != null) {
                grailsHome = root + File.separator + "grails";
            }
        }

        if (grailsHome == null) {
            deploymentContext.getLogger().log(Level.SEVERE, Messages.format(Messages.DEPLOYER_GRAILSBUNDLE_NOTFOUND));
            return null;
        }

        File grailsLoc = new File(grailsHome + File.separator + "lib" + File.separator + "glassfish-grails.jar");
        if (!grailsLoc.exists()) {
            //now try to look for grails IPS package
            if (root != null) {
                grailsHome = root + File.separator + "grails";
            }
            grailsLoc = new File(grailsHome + File.separator + "lib"+ File.separator + "glassfish-grails.jar");

            if (!grailsLoc.exists()) {
                deploymentContext.getLogger().log(Level.WARNING, Messages.format(Messages.DEPLOYER_GRAILS_GLASSFISHJAR_NOTFOUND, grailsLoc.toString()));
                return null;
            }
            ReadableArchive archive = deploymentContext.getSource();
            if(!hasGrailsJars(archive) &&
                    !librariesHasGrailsJar(deploymentContext.getCommandParameters(DeployCommandParameters.class).libraries)){
                throw new RuntimeException(Messages.format(Messages.DEPLOYER_GRAILS_DEPLOY, archive.getName()+".war", grailsLoc.getAbsolutePath()));
            }
        }

        return null;

    }

    private boolean librariesHasGrailsJar(String path){
        if(path == null)
            return false;
        String [] jars = path.split(",");
        for(String jar : jars){
            if(jar.trim().endsWith("glassfish-grails.jar"))
                return true;
        }
        return false;
    }

    private boolean hasGrailsJars(ReadableArchive archive){
        Enumeration<String> entries = archive.entries("WEB-INF/lib");
        List<String> jarlist = new ArrayList<String>();
        while (entries.hasMoreElements()) {
            String entryName = entries.nextElement();
            //For now check for some core jars presence in the WAR file
            //and if found then definitely it is not a shared-war file
            if (entryName.endsWith(".jar")){
                jarlist.add(entryName);
            }

        }
        boolean present = false;
        for(String jar : grailsDependencies){
            for(String str : jarlist){
                if(str.contains(jar)){
                    present = true;
                    break;
                }
            }
        }
        return present;
    }

    private final String[] grailsDependencies = {"grails-web", "grails-core", "grails-spring", "hibernet", "xercesImpl.jar"};

    public boolean prepare(DeploymentContext deploymentContext) {
        loadMetaData(this.getClass(), deploymentContext);
        return true;
    }

    public GrailsApplicationContainer load(GrailsContainer grailsContainer, DeploymentContext deploymentContext) {
        
        return new GrailsApplicationContainer();
    }

    public void unload(GrailsApplicationContainer grailsApplicationContainer, DeploymentContext deploymentContext) {
    }

    public void clean(DeploymentContext deploymentContext) {

    }
}
